import { ApiProperty } from '@nestjs/swagger';
import { AllowanceChangeRequest } from '~/allowance/entities';
import { AllowanceChangeRequestStatus } from '~/allowance/enums';
import { JuniorResponseDto } from '~/junior/dtos/response';

export class AllowanceChangeRequestResponseDto {
  @ApiProperty({ example: 'd641bb71-2e7c-4e62-96fa-2785f0a651c6' })
  id!: string;

  @ApiProperty({ example: AllowanceChangeRequestStatus.APPROVED })
  status!: AllowanceChangeRequestStatus;

  @ApiProperty({ example: 'Allowance name' })
  name!: string;

  @ApiProperty({ example: '100' })
  oldAmount!: number;

  @ApiProperty({ example: '200' })
  newAmount!: number;

  @ApiProperty({ example: 'Some reason' })
  reason!: string;

  @ApiProperty({ example: 'd641bb71-2e7c-4e62-96fa-2785f0a651c6' })
  allowanceId!: string;

  @ApiProperty({ type: JuniorResponseDto })
  junior!: JuniorResponseDto;

  @ApiProperty({ example: new Date() })
  createdAt!: Date;

  constructor(allowanceChangeRequest: AllowanceChangeRequest) {
    this.id = allowanceChangeRequest.id;
    this.status = allowanceChangeRequest.status;
    this.name = allowanceChangeRequest.allowance.name;
    this.oldAmount = allowanceChangeRequest.allowance.amount;
    this.newAmount = allowanceChangeRequest.amount;
    this.reason = allowanceChangeRequest.reason;
    this.allowanceId = allowanceChangeRequest.allowanceId;
    this.junior = new JuniorResponseDto(allowanceChangeRequest.allowance.junior);
    this.createdAt = allowanceChangeRequest.createdAt;
  }
}
