import {
  Column,
  CreateDateColumn,
  DeleteDateColumn,
  Entity,
  JoinColumn,
  ManyToOne,
  OneToMany,
  PrimaryGeneratedColumn,
  UpdateDateColumn,
} from 'typeorm';
import { Guardian } from '~/guardian/entities/guradian.entity';
import { Junior } from '~/junior/entities';
import { AllowanceFrequency, AllowanceType } from '../enums';
import { AllowanceChangeRequest } from './allowance-change-request.entity';

@Entity('allowances')
export class Allowance {
  @PrimaryGeneratedColumn('uuid')
  id!: string;

  @Column({ type: 'varchar', length: 255, name: 'name' })
  name!: string;

  @Column({
    type: 'decimal',
    precision: 10,
    scale: 2,
    name: 'amount',
    transformer: { to: (value: number) => value, from: (value: string) => parseFloat(value) },
  })
  amount!: number;

  @Column({ type: 'varchar', length: 255, name: 'frequency' })
  frequency!: AllowanceFrequency;

  @Column({ type: 'varchar', length: 255, name: 'type' })
  type!: AllowanceType;

  @Column({ type: 'timestamp with time zone', name: 'start_date' })
  startDate!: Date;

  @Column({ type: 'timestamp with time zone', name: 'end_date', nullable: true })
  endDate?: Date;

  @Column({ type: 'int', name: 'number_of_transactions', nullable: true })
  numberOfTransactions?: number;

  @Column({ type: 'uuid', name: 'guardian_id' })
  guardianId!: string;

  @Column({ type: 'uuid', name: 'junior_id' })
  juniorId!: string;

  @ManyToOne(() => Guardian, (guardian) => guardian.allowances)
  @JoinColumn({ name: 'guardian_id' })
  guardian!: Guardian;

  @ManyToOne(() => Junior, (junior) => junior.allowances)
  @JoinColumn({ name: 'junior_id' })
  junior!: Junior;

  @OneToMany(() => AllowanceChangeRequest, (changeRequest) => changeRequest.allowance)
  changeRequests!: AllowanceChangeRequest[];

  @CreateDateColumn({ name: 'created_at', type: 'timestamp with time zone', default: () => 'CURRENT_TIMESTAMP' })
  createdAt!: Date;

  @UpdateDateColumn({ name: 'updated_at', type: 'timestamp with time zone', default: () => 'CURRENT_TIMESTAMP' })
  updatedAt!: Date;

  @DeleteDateColumn({ name: 'deleted_at', type: 'timestamp with time zone', nullable: true })
  deletedAt?: Date;
}
