import { forwardRef, Module } from '@nestjs/common';
import { JwtModule } from '@nestjs/jwt';
import { TypeOrmModule } from '@nestjs/typeorm';
import { CustomerModule } from '~/customer/customer.module';
import { JuniorModule } from '~/junior/junior.module';
import { AuthController } from './controllers';
import { Device, User } from './entities';
import { DeviceRepository, UserRepository } from './repositories';
import { AuthService, DeviceService } from './services';
import { UserService } from './services/user.service';
import { AccessTokenStrategy } from './strategies';

@Module({
  imports: [
    TypeOrmModule.forFeature([User, Device]),
    JwtModule.register({}),
    forwardRef(() => CustomerModule),
    forwardRef(() => JuniorModule),
  ],
  providers: [AuthService, UserRepository, UserService, DeviceService, DeviceRepository, AccessTokenStrategy],
  controllers: [AuthController],
  exports: [UserService, DeviceService],
})
export class AuthModule {}
