import {
  BaseEntity,
  Column,
  CreateDateColumn,
  Entity,
  OneToMany,
  OneToOne,
  PrimaryGeneratedColumn,
  UpdateDateColumn,
} from 'typeorm';
import { Notification } from '~/common/modules/notification/entities';
import { Otp } from '~/common/modules/otp/entities';
import { Customer } from '~/customer/entities/customer.entity';
import { Roles } from '../enums';
import { Device } from './device.entity';

@Entity('users')
export class User extends BaseEntity {
  @PrimaryGeneratedColumn('uuid')
  id!: string;

  @Column('varchar', { length: 255, nullable: true, name: 'email' })
  email!: string;

  @Column('varchar', { length: 255, name: 'phone_number' })
  phoneNumber!: string;

  @Column('varchar', { length: 10, name: 'country_code' })
  countryCode!: string;

  @Column('varchar', { length: 255, name: 'password', nullable: true })
  password!: string;

  @Column('varchar', { length: 255, name: 'salt', nullable: true })
  salt!: string;

  @Column('varchar', { length: 255, nullable: true, name: 'google_id' })
  googleId!: string;

  @Column('varchar', { length: 255, nullable: true, name: 'apple_id' })
  appleId!: string;

  @Column('boolean', { default: false, name: 'is_profile_completed' })
  isProfileCompleted!: boolean;

  @Column('text', { nullable: true, array: true, name: 'roles' })
  roles!: Roles[];

  @OneToMany(() => Otp, (otp) => otp.user)
  otp!: Otp[];

  @OneToOne(() => Customer, (customer) => customer.user, { cascade: true, eager: true })
  customer!: Customer;

  @OneToMany(() => Device, (device) => device.user)
  devices!: Device[];

  @OneToMany(() => Notification, (notification) => notification.user)
  notifications!: Notification[];

  @CreateDateColumn({ name: 'created_at', type: 'timestamp with time zone' })
  createdAt!: Date;

  @UpdateDateColumn({ name: 'updated_at', type: 'timestamp with time zone' })
  updatedAt!: Date;

  get isPasswordSet(): boolean {
    return !!this.password;
  }
}
