import { Module } from '@nestjs/common';
import { ConfigService } from '@nestjs/config';
import { TypeOrmModule } from '@nestjs/typeorm';
import { TwilioModule } from 'nestjs-twilio';
import { AuthModule } from '~/auth/auth.module';
import { buildTwilioOptions } from '~/core/module-options';
import { NotificationsController } from './controllers';
import { Notification } from './entities';
import { NotificationsRepository } from './repositories';
import { FirebaseService } from './services/firebase.service';
import { NotificationsService } from './services/notifications.service';
import { TwilioService } from './services/twilio.service';
@Module({
  imports: [
    TypeOrmModule.forFeature([Notification]),
    AuthModule,
    TwilioModule.forRootAsync({
      useFactory: buildTwilioOptions,
      inject: [ConfigService],
    }),
  ],
  providers: [NotificationsService, FirebaseService, NotificationsRepository, TwilioService],
  exports: [NotificationsService],
  controllers: [NotificationsController],
})
export class NotificationModule {}
