import { Injectable } from '@nestjs/common';
import { DeviceService } from '~/auth/services';
import { PageOptionsRequestDto } from '~/core/dtos';
import { Notification } from '../entities';
import { NotificationsRepository } from '../repositories';
import { FirebaseService } from './firebase.service';
import { TwilioService } from './twilio.service';

@Injectable()
export class NotificationsService {
  constructor(
    private readonly deviceService: DeviceService,
    private readonly firebaseService: FirebaseService,
    private readonly notificationRepository: NotificationsRepository,
    private readonly twilioService: TwilioService,
  ) {}

  async sendPushNotification(userId: string, title: string, body: string) {
    // Get the device tokens for the user

    const tokens = await this.deviceService.getTokens(userId);

    if (!tokens.length) {
      return;
    }
    // Send the notification
    return this.firebaseService.sendNotification(tokens, title, body);
  }

  async sendSMS(to: string, body: string) {
    await this.twilioService.sendSMS(to, body);
  }

  async getNotifications(userId: string, pageOptionsDto: PageOptionsRequestDto) {
    const [[notifications, count], unreadCount] = await Promise.all([
      this.notificationRepository.getNotifications(userId, pageOptionsDto),
      this.notificationRepository.getUnreadNotificationsCount(userId),
    ]);

    return { notifications, count, unreadCount };
  }

  createNotification(notification: Partial<Notification>) {
    return this.notificationRepository.createNotification(notification);
  }

  markAsRead(userId: string) {
    return this.notificationRepository.markAsRead(userId);
  }
}
