import { Injectable, Logger } from '@nestjs/common';
import { ConfigService } from '@nestjs/config';
import { TwilioService as TwilioApiService } from 'nestjs-twilio';
import { Environment } from '~/core/enums';
@Injectable()
export class TwilioService {
  private logger = new Logger(TwilioService.name);
  constructor(private readonly twilioService: TwilioApiService, private readonly configService: ConfigService) {}
  private from = this.configService.getOrThrow('TWILIO_PHONE_NUMBER');
  sendSMS(to: string, body: string) {
    if (this.configService.get('NODE_ENV') === Environment.DEV) {
      this.logger.log(`Skipping SMS sending in DEV environment. Message: ${body} to: ${to}`);
      return;
    }
    return this.twilioService.client.messages.create({
      body,
      to,
      from: this.from,
    });
  }
}
