import { ApiProperty, ApiPropertyOptional } from '@nestjs/swagger';
import { Customer } from '~/customer/entities';
import { DocumentMetaResponseDto } from '~/document/dtos/response';
import { NotificationSettingsResponseDto } from './notification-settings.response.dto';

export class CustomerResponseDto {
  @ApiProperty()
  id!: string;

  @ApiProperty()
  customerStatus!: string;

  @ApiProperty()
  rejectionReason!: string;

  @ApiProperty()
  firstName!: string;

  @ApiProperty()
  lastName!: string;

  @ApiProperty()
  dateOfBirth!: Date;

  @ApiProperty()
  nationalId!: string;

  @ApiProperty()
  nationaIdExpiry!: Date;

  @ApiProperty()
  countryOfResidence!: string;

  @ApiProperty()
  sourceOfIncome!: string;

  @ApiProperty()
  profession!: string;

  @ApiProperty()
  professionType!: string;

  @ApiProperty()
  isPep!: boolean;

  @ApiProperty()
  gender!: string;

  @ApiProperty()
  isJunior!: boolean;

  @ApiProperty()
  isGuardian!: boolean;

  @ApiProperty()
  notificationSettings!: NotificationSettingsResponseDto;

  @ApiPropertyOptional({ type: DocumentMetaResponseDto })
  profilePicture!: DocumentMetaResponseDto | null;

  constructor(customer: Customer) {
    this.id = customer.id;
    this.customerStatus = customer.customerStatus;
    this.rejectionReason = customer.rejectionReason;
    this.firstName = customer.firstName;
    this.lastName = customer.lastName;
    this.dateOfBirth = customer.dateOfBirth;
    this.nationalId = customer.nationalId;
    this.nationaIdExpiry = customer.nationaIdExpiry;
    this.countryOfResidence = customer.countryOfResidence;
    this.sourceOfIncome = customer.sourceOfIncome;
    this.profession = customer.profession;
    this.professionType = customer.professionType;
    this.isPep = customer.isPep;
    this.gender = customer.gender;
    this.isJunior = customer.isJunior;
    this.isGuardian = customer.isGuardian;
    this.notificationSettings = new NotificationSettingsResponseDto(customer.notificationSettings);
    this.profilePicture = customer.profilePicture ? new DocumentMetaResponseDto(customer.profilePicture) : null;
  }
}
