import { Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { FindOptionsWhere, Repository } from 'typeorm';
import { User } from '~/auth/entities';
import { Roles } from '~/auth/enums';
import { UpdateNotificationsSettingsRequestDto } from '../dtos/request';
import { Customer } from '../entities';
import { CustomerNotificationSettings } from '../entities/customer-notification-settings.entity';

@Injectable()
export class CustomerRepository {
  constructor(@InjectRepository(Customer) private readonly customerRepository: Repository<Customer>) {}

  updateCustomer(id: string, data: Partial<Customer>) {
    return this.customerRepository.update(id, data);
  }

  findOne(where: FindOptionsWhere<Customer>) {
    return this.customerRepository.findOne({ where, relations: ['profilePicture'] });
  }

  createCustomer(customerData: Partial<Customer>, user: User) {
    return this.customerRepository.save(
      this.customerRepository.create({
        ...customerData,
        id: user.id,
        user,
        isGuardian: user.roles.includes(Roles.GUARDIAN),
        isJunior: user.roles.includes(Roles.JUNIOR),
      }),
    );
  }

  updateNotificationSettings(customer: Customer, body: UpdateNotificationsSettingsRequestDto) {
    customer.notificationSettings = CustomerNotificationSettings.create({
      ...customer.notificationSettings,
      ...body,
    });
    return this.customerRepository.save(customer);
  }
}
