import { BadRequestException, forwardRef, Inject, Injectable } from '@nestjs/common';
import { User } from '~/auth/entities';
import { DeviceService } from '~/auth/services';
import { OciService } from '~/document/services';
import { UpdateCustomerRequestDto, UpdateNotificationsSettingsRequestDto } from '../dtos/request';
import { Customer } from '../entities';
import { CustomerRepository } from '../repositories/customer.repository';

@Injectable()
export class CustomerService {
  constructor(
    private readonly customerRepository: CustomerRepository,
    private readonly ociService: OciService,
    @Inject(forwardRef(() => DeviceService)) private readonly deviceService: DeviceService,
  ) {}
  async updateNotificationSettings(userId: string, data: UpdateNotificationsSettingsRequestDto, deviceId: string) {
    const customer = await this.findCustomerById(userId);

    const notificationSettings = (await this.customerRepository.updateNotificationSettings(customer, data))
      .notificationSettings;

    if (data.isPushEnabled && deviceId) {
      await this.deviceService.updateDevice(deviceId, {
        fcmToken: data.fcmToken,
        userId: userId,
      });
    }

    return notificationSettings;
  }

  async updateCustomer(userId: string, data: UpdateCustomerRequestDto): Promise<Customer> {
    await this.customerRepository.updateCustomer(userId, data);
    return this.findCustomerById(userId);
  }

  createCustomer(customerData: Partial<Customer>, user: User) {
    return this.customerRepository.createCustomer(customerData, user);
  }

  async findCustomerById(id: string) {
    const customer = await this.customerRepository.findOne({ id });

    if (!customer) {
      throw new BadRequestException('CUSTOMER.NOT_FOUND');
    }

    if (customer.profilePicture) {
      customer.profilePicture.url = await this.ociService.generatePreSignedUrl(customer.profilePicture);
    }
    return customer;
  }
}
