import {
  BaseEntity,
  Column,
  CreateDateColumn,
  Entity,
  Index,
  JoinColumn,
  ManyToOne,
  PrimaryGeneratedColumn,
  UpdateDateColumn,
} from 'typeorm';
import { Junior } from './junior.entity';

@Entity('junior_registration_tokens')
export class JuniorRegistrationToken extends BaseEntity {
  @PrimaryGeneratedColumn('uuid')
  id!: string;

  @Column({ type: 'varchar', length: 255, name: 'token', unique: true })
  @Index()
  token!: string;

  @Column({ type: 'boolean', default: false, name: 'is_used' })
  isUsed!: boolean;

  @Column({ type: 'timestamp', name: 'expiry_date' })
  expiryDate!: Date;

  @Column({ type: 'uuid', name: 'junior_id' })
  juniorId!: string;

  @ManyToOne(() => Junior, (junior) => junior.registrationTokens)
  @JoinColumn({ name: 'junior_id' })
  junior!: Junior;

  @UpdateDateColumn({ type: 'timestamp', default: () => 'CURRENT_TIMESTAMP', name: 'updated_at' })
  updatedAt!: Date;

  @CreateDateColumn({ type: 'timestamp', default: () => 'CURRENT_TIMESTAMP', name: 'created_at' })
  createdAt!: Date;
}
