import { BadRequestException, Injectable } from '@nestjs/common';
import { JuniorTokenRepository } from '../repositories';
import { QrcodeService } from './qrcode.service';

@Injectable()
export class JuniorTokenService {
  constructor(
    private readonly juniorTokenRepository: JuniorTokenRepository,
    private readonly qrCodeService: QrcodeService,
  ) {}
  async generateToken(juniorId: string): Promise<string> {
    const tokenEntity = await this.juniorTokenRepository.generateToken(juniorId);
    return this.qrCodeService.generateQrCode(tokenEntity.token);
  }

  async validateToken(token: string) {
    const tokenEntity = await this.juniorTokenRepository.findByToken(token);

    if (!tokenEntity) {
      throw new BadRequestException('TOKEN.INVALID');
    }

    if (tokenEntity.expiryDate < new Date()) {
      throw new BadRequestException('TOKEN.EXPIRED');
    }

    return tokenEntity.juniorId;
  }

  invalidateToken(token: string) {
    return this.juniorTokenRepository.invalidateToken(token);
  }
}
