import {
  Column,
  CreateDateColumn,
  Entity,
  JoinColumn,
  ManyToOne,
  PrimaryGeneratedColumn,
  UpdateDateColumn,
} from 'typeorm';
import { Guardian } from '~/guardian/entities/guradian.entity';
import { Junior } from '~/junior/entities';
import { MoneyRequestFrequency, MoneyRequestStatus } from '../enums';

@Entity('money_requests')
export class MoneyRequest {
  @PrimaryGeneratedColumn('uuid')
  id!: string;

  @Column({ type: 'decimal', precision: 10, scale: 3, name: 'requested_amount' })
  requestedAmount!: number;

  @Column({ type: 'varchar', name: 'message' })
  message!: string;

  @Column({ type: 'varchar', name: 'frequency', default: MoneyRequestFrequency.ONE_TIME })
  frequency!: MoneyRequestFrequency;

  @Column({ type: 'varchar', name: 'status', default: MoneyRequestStatus.PENDING })
  status!: MoneyRequestStatus;

  @Column({ type: 'timestamp with time zone', name: 'reviewed_at', nullable: true })
  reviewedAt!: Date;

  @Column({ type: 'timestamp with time zone', name: 'end_date', nullable: true })
  endDate!: Date | null;

  @Column({ type: 'uuid', name: 'requester_id' })
  requesterId!: string;

  @Column({ type: 'uuid', name: 'reviewer_id' })
  reviewerId!: string;

  @ManyToOne(() => Junior, (junior) => junior.moneyRequests)
  @JoinColumn({ name: 'requester_id' })
  requester!: Junior;

  @ManyToOne(() => Guardian, (guardian) => guardian.moneyRequests)
  @JoinColumn({ name: 'reviewer_id' })
  reviewer!: Guardian;

  @CreateDateColumn({ type: 'timestamp with time zone', name: 'created_at' })
  createdAt!: Date;

  @UpdateDateColumn({ type: 'timestamp with time zone', name: 'updated_at' })
  updatedAt!: Date;
}
