import { Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Repository } from 'typeorm';
import { CreateMoneyRequestRequestDto, MoneyRequestsFiltersRequestDto } from '../dtos/request';
import { MoneyRequest } from '../entities';
import { MoneyRequestStatus } from '../enums';
const ONE = 1;
@Injectable()
export class MoneyRequestsRepository {
  constructor(@InjectRepository(MoneyRequest) private readonly moneyRequestRepository: Repository<MoneyRequest>) {}

  createMoneyRequest(requesterId: string, reviewerId: string, body: CreateMoneyRequestRequestDto) {
    return this.moneyRequestRepository.save(
      this.moneyRequestRepository.create({
        requesterId,
        reviewerId,
        requestedAmount: body.requestedAmount,
        message: body.message,
        endDate: body.endDate,
        frequency: body.frequency,
      }),
    );
  }

  findMoneyRequestById(moneyRequestId: string, reviewerId?: string) {
    return this.moneyRequestRepository.findOne({
      where: { id: moneyRequestId, reviewerId },
      relations: ['requester', 'requester.customer', 'requester.customer.profilePicture'],
    });
  }

  findMoneyRequests(reviewerId: string, filters: MoneyRequestsFiltersRequestDto) {
    const query = this.moneyRequestRepository.createQueryBuilder('moneyRequest');
    query.leftJoinAndSelect('moneyRequest.requester', 'requester');
    query.leftJoinAndSelect('requester.customer', 'customer');
    query.leftJoinAndSelect('customer.profilePicture', 'profilePicture');
    query.orderBy('moneyRequest.createdAt', 'DESC');
    query.where('moneyRequest.reviewerId = :reviewerId', { reviewerId });
    query.andWhere('moneyRequest.status = :status', { status: filters.status });
    query.skip((filters.page - ONE) * filters.size);
    query.take(filters.size);

    return query.getManyAndCount();
  }

  updateMoneyRequestStatus(moneyRequestId: string, reviewerId: string, status: MoneyRequestStatus) {
    return this.moneyRequestRepository.update({ id: moneyRequestId, reviewerId }, { status, reviewedAt: new Date() });
  }
}
