import { BadRequestException, Injectable } from '@nestjs/common';
import { createCategoryRequestDto } from '../dtos/request';
import { Category } from '../entities';
import { CategoryType } from '../enums';
import { CategoryRepository } from '../repositories';

@Injectable()
export class CategoryService {
  constructor(private readonly categoryRepository: CategoryRepository) {}

  createCustomCategory(juniorId: string, body: createCategoryRequestDto) {
    return this.categoryRepository.createCustomCategory(juniorId, body);
  }

  async findCategoryByIds(categoryIds: string[], juniorId: string) {
    const categories = await this.categoryRepository.findCategoryById(categoryIds, juniorId);

    if (categories.length !== categoryIds.length) {
      throw new BadRequestException('CATEGORY.NOT_FOUND');
    }
    return categories;
  }

  async findCategories(juniorId: string): Promise<{ globalCategories: Category[]; customCategories: Category[] }> {
    const categories = await this.categoryRepository.findCategories(juniorId);

    const globalCategories = categories.filter((category) => category.type === CategoryType.GLOBAL);
    const customCategories = categories.filter((category) => category.type === CategoryType.CUSTOM);

    return {
      globalCategories,
      customCategories,
    };
  }
}
